/******************************************************************************************************************
TITLE       -  PIVOTS LOWER

FILE        -  pivots lower.mp4

DESCRIPTION -  displays the Pivot, lower support & resistance lines, PP,S1,S2,S3,M0,M1,M2 on the Screen, as well as 
               their label.  The "Bain Day" type will be printed at the Pivot Point line.

USAGE       -  refer to PL.pdf

REVISIONS   -  Written by: CrushD
            -  1.0   14/2/06  Heavily Modified by: Mark A Flexhaug, pipaholic@shaw.ca
            -  2.0   17/2/06  Added MMTS and market opening indicators
            -  2.1   22/2/06  fixed BainDay type bug & S2 label 
            -  2.2   23/02/06 remove opening lines on intervals >60
            
ISSUES      -  pivots do not work for periods > 15 minutes
******************************************************************************************************************/

#property indicator_chart_window
#property indicator_buffers 7

#property indicator_color1 Crimson
#property indicator_color2 Blue
#property indicator_color3 Crimson
#property indicator_color4 Blue
#property indicator_color5 Crimson
#property indicator_color6 Navy
#property indicator_color7 Navy

//---- input parameters\
extern   bool  ShowOpens   = true,
               ShowLabels  = true;
extern string StartTime = "06:00";
extern string EndTime = "05:55";
extern int  timeZone = 0;
extern string  NewsTime = "15:15";
extern double  Slush = 0.0;
extern bool    RunFast     = true;

double p, s1, s2, s3, r1, r2, r3, m0, m1, m2, m3, m4, m5;
double pivotOpen=0;
double pivotHigh=0;
double pivotLow=0;
double pivotClose=0;
bool  pivotStarted=false;
bool  pivotCompleted=false;

// buffers
double PivotPoint[];
double M2[];
double S1[];
double M1[];
double S2[];
double M0[];
double S3[];

// Some definitions
int   FontSize    = 12;
int   timeAdjust  = 0;
double   thick;   
int   NO       = 0;
int   YES      = 1;
int   run      = NULL;
int   mmtsIndex;
int   vInt     = NULL;
int   nCount   = NULL;
int   BainDay;

//+------------------------------------------------------------------+
//| Custom indicator initialization function |
//+------------------------------------------------------------------+
int init()
{
//---- indicators
IndicatorShortName("Pivot Lower");

SetIndexStyle(0, DRAW_LINE,STYLE_SOLID,3);
SetIndexBuffer(0, PivotPoint);
SetIndexLabel(0, "Pivot Point");

SetIndexStyle(1, DRAW_LINE, STYLE_DASH, 1);
SetIndexBuffer(1, M2);
SetIndexLabel(1, "M2");

SetIndexStyle(2, DRAW_LINE,STYLE_SOLID,1);
SetIndexBuffer(2, S1);
SetIndexLabel(2, "S1");

SetIndexStyle(3, DRAW_LINE, STYLE_DASH, 1);
SetIndexBuffer(3, M1);
SetIndexLabel(3, "M1");

SetIndexStyle(4,DRAW_LINE,STYLE_SOLID,1);
SetIndexBuffer(4, S2);
SetIndexLabel(4, "S2");

SetIndexStyle(5, DRAW_LINE, STYLE_DASH, 1);
SetIndexBuffer(5, M0);
SetIndexLabel(5, "M0");

SetIndexStyle(6, DRAW_LINE,STYLE_SOLID,1);
SetIndexBuffer(6, S3);
SetIndexLabel(6, "S3");

switch(Period())
{
   case 1:
      vInt     = 1;
      nCount   = 30;
      break;
   case 5:
      vInt     = 5;
      nCount   = 6;
      break;
   case 15:
      vInt     = 15;
      nCount   = 2;
      break;
   case 30:
      vInt     = 30;
      nCount   = 1;
      break;
   case 60:
   case 240:
   case 1440:
      vInt     = 0;
      break;
}
return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function |
//+------------------------------------------------------------------+
int deinit()
{
      ObjectDelete("euro");
      ObjectDelete("london");
      ObjectDelete("ny");
      ObjectDelete("eleven");
      ObjectDelete("rollover");
      ObjectDelete("sydney");
      ObjectDelete("asian");
      ObjectDelete("hk");  
      ObjectDelete("BT");
      ObjectDelete("S1");
      ObjectDelete("S2");
      ObjectDelete("S3");
      ObjectDelete("M2");
      ObjectDelete("M1");
      ObjectDelete("M0");      
return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function |
//+------------------------------------------------------------------+
int start()
{
   int   i,limit,
         extra    = 2500;
   int   BainType = NULL;
   int   vYear    = Year();
   int   vMonth   = Month();
   int   vDay     = Day();
   int   vHour    = Hour();
   int   vMinute  = Minute();
   datetime vNow  = NULL;

   vNow  = StrToTime((vYear+"."+vMonth+"."+vDay+" "+vHour+":"+(vMinute-Period())));

   if(RunFast) limit = 10 * 24*(60/Period());
   else        limit = Bars;
   for(i=limit; i>=0; i--)
   {
//      if ((Period() == 1) || (Period() <= 60)) BainType = calcPivots(i);
      BainType = calcPivots(i);      
      if(ShowOpens)  doOpens(i);     
      doEntry(i);
      PivotPoint[i] = p;
      S1[i] = s1;
      S2[i] = s2;
      S3[i] = s3;
      M2[i] = m2;      
      M1[i] = m1;      
      M0[i] = m0;
   }
// print the PRS line labels just under the line   

   if(ShowLabels)
   {
      ObjectDelete("BT");
      ObjectDelete("S1");
      ObjectDelete("S2");
      ObjectDelete("S3");
      ObjectDelete("M2");
      ObjectDelete("M1");
      ObjectDelete("M0");

      ObjectCreate("BT", OBJ_TEXT, 0, vNow+(extra*3), p);
      if(BainDay == 1)  ObjectSetText("BT", "M2/M4   ", FontSize+4, "Arial", Yellow);
      else              ObjectSetText("BT", "M1/M3   ", FontSize+4, "Arial", Yellow);
   
      ObjectCreate("S1", OBJ_TEXT, 0, vNow+extra, s1);
      ObjectSetText("S1", "S1", FontSize, "Arial", Navy);
      ObjectCreate("S2", OBJ_TEXT, 0, vNow+extra, s2);
      ObjectSetText("S2", "S2", FontSize, "Arial", Navy);
      ObjectCreate("S3", OBJ_TEXT, 0, vNow+extra, s3);
      ObjectSetText("S3", "S3", FontSize, "Arial", Navy);
      ObjectCreate("M2", OBJ_TEXT, 0, vNow+extra, m2);
      ObjectSetText("M2", "M2", FontSize, "Arial", Navy);
      ObjectCreate("M1", OBJ_TEXT, 0, vNow+extra, m1);
      ObjectSetText("M1", "M1", FontSize, "Arial", Navy);
      ObjectCreate("M0", OBJ_TEXT, 0, vNow+extra, m0);
      ObjectSetText("M0", "M0", FontSize, "Arial", Navy);
   }
}

int calcPivots(int curBar)
{

   string barTime;   
   double projHigh, projLow;
   
   barTime = TimeToStr(Time[curBar], TIME_MINUTES);
       
   if (barTime == StartTime)
   {
     pivotOpen = Open[curBar];
     pivotLow = Low[curBar];
     pivotHigh = High[curBar];
     pivotStarted=true;
   }

   if (pivotStarted && barTime == EndTime)
   {   
     pivotClose = Close[curBar];
     pivotCompleted=true; 
   }

   if (pivotStarted && pivotHigh < High[curBar])
   {
      pivotHigh=High[curBar];
   } 

   if (pivotStarted && pivotLow > Low[curBar]) 
   {
      pivotLow=Low[curBar];
   }  

   if (pivotCompleted)
   {
      p = (pivotHigh + pivotLow + pivotClose)/3;
      s1 = (2 * p) - pivotHigh;
      r1 = (2 * p) - pivotLow;
      s2 = p - r1 + s1;
      r2 = p -s1 + r1;
      s3 = pivotLow - (2*(pivotHigh - p));
      r3 = pivotHigh + (2*(p - pivotLow));
      m0 = (s2 + s3)/2;
      m1 = (s1 + s2)/2;
      m2 = (p + s1)/2;
      m3 = (p + r1)/2; 
      m4 = (r1 + r2)/2;
      m5 = (r2 + r3)/2;      
      
      pivotStarted = false;
      pivotCompleted = false; 

      if(pivotClose > pivotOpen)
      {  
         projHigh = m4;
         projLow = m2;  
         BainDay  = 1;
         return(1);
      }
      else
      {
         projHigh = m3;
         projLow = m1;
         BainDay = 0;
         return(0);
      }

   }

}

int doOpens(int curBar)
{
   string   barTime;
   string   euroOpen    = "06:00";
   string   londonOpen  = "07:00";
   string   nyOpen      = "12:00";
   string   eleven      = "15:00";
   string   rollover    = "21:00";
   string   sydneyOpen  = "22:00";
   string   asianOpen   = "23:00";
   string   hkOpen      = "00:00"; 
     
   datetime vWhen;

   barTime = TimeToStr(Time[curBar], TIME_MINUTES);
   vWhen = StrToTime(barTime);

   if (Period() > 60)
   {
      ObjectDelete("euro");
      ObjectDelete("london");
      ObjectDelete("ny");
      ObjectDelete("eleven");
      ObjectDelete("rollover");
      ObjectDelete("sydney");
      ObjectDelete("asian");
      ObjectDelete("hk");                        
      return(0);
   }
            
   if(barTime == euroOpen)             // european session
   {
      ObjectDelete("euro");
      ObjectCreate("euro",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("euro",OBJPROP_COLOR,DeepSkyBlue);
      ObjectSet("euro",OBJPROP_WIDTH,1);
      ObjectSet("euro",OBJPROP_STYLE,STYLE_SOLID);                  
   }
   else if (barTime == londonOpen)    // london session
   {
      ObjectDelete("london");
      ObjectCreate("london",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("london",OBJPROP_COLOR,DeepSkyBlue);
      ObjectSet("london",OBJPROP_WIDTH,1);
      ObjectSet("london",OBJPROP_STYLE,STYLE_SOLID);                  
   }
   else if (barTime == nyOpen)       // ny session
   {
      ObjectDelete("ny");
      ObjectCreate("ny",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("ny",OBJPROP_COLOR,DeepSkyBlue);
      ObjectSet("ny",OBJPROP_WIDTH,1);
      ObjectSet("ny",OBJPROP_STYLE,STYLE_SOLID);                  
   }
      else if (barTime == eleven)    // eleven 
   {
      ObjectDelete("eleven");
      ObjectCreate("eleven",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("eleven",OBJPROP_COLOR,Orange);
      ObjectSet("eleven",OBJPROP_WIDTH,1);
      ObjectSet("eleven",OBJPROP_STYLE,STYLE_SOLID);               
   }
      else if (barTime == rollover)    // eleven 
   {
      ObjectDelete("rollover");
      ObjectCreate("rollover",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("rollover",OBJPROP_COLOR,Orange);
      ObjectSet("rollover",OBJPROP_WIDTH,1);
      ObjectSet("rollover",OBJPROP_STYLE,STYLE_SOLID);               
   }
      else if (barTime == sydneyOpen) // sydney session
   {
      ObjectDelete("sydney");
      ObjectCreate("sydney",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("sydney",OBJPROP_COLOR,DeepSkyBlue);
      ObjectSet("sydney",OBJPROP_WIDTH,1);
      ObjectSet("sydney",OBJPROP_STYLE,STYLE_SOLID);                  
   }
      else if (barTime == asianOpen) // asian session
   {
      ObjectDelete("asian");
      ObjectCreate("asian",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("asian",OBJPROP_COLOR,DeepSkyBlue);
      ObjectSet("asian",OBJPROP_WIDTH,1);
      ObjectSet("asian",OBJPROP_STYLE,STYLE_SOLID);                  
   }
      else if (barTime == hkOpen)   // hong kong session
   {
      ObjectDelete("hk");  
      ObjectCreate("hk",OBJ_VLINE,0,vWhen,NULL);
      ObjectSet("hk",OBJPROP_COLOR,DeepSkyBlue);
      ObjectSet("hk",OBJPROP_WIDTH,1);
      ObjectSet("hk",OBJPROP_STYLE,STYLE_SOLID);                  
   } 
}
/*
int nrats(int curBar)
{


   int      i,j;
   int      limit;
   double   mmtsLow;
   double   mmtsLow1;
   double   mmtsHigh;
   double   mmtsHigh1;
   string   barTime;
   string   barDate;
   string   barEnd;
   string   mmtsTime = "14:00";
   string   endTime  = "17:00";
   datetime vStart;
   datetime vWhen;   
   datetime vEnd;

   return(0);
   if(Symbol() == "EURUSD") 
   {
      thick = 0.0001;
   }
   else if (Symbol() == "GBPUSD")
   {
      thick = 0.0001;
   }
   else if (Symbol() == "USDCHF")
   {
      thick = 0.0001;
   }
   else    return(0);
         
   barTime = TimeToStr(Time[curBar], TIME_MINUTES);

   int counted_bars = IndicatorCounted();
   //---- check for possible errors
   if (counted_bars<0) return(-1);
   //---- last counted bar will be recounted
   if (counted_bars>0) counted_bars--;
   limit = Bars - counted_bars;

   for(i=limit; i>=0; i--)
   {
      if(barTime == NewsTime) 
      {    
         if (vInt != 0)
         {
            vWhen    = StrToTime(barTime);
            vEnd     = StrToTime(endTime);

            mmtsLow  = iLow(NULL,vInt,curBar+1);
            mmtsHigh  = iHigh(NULL,vInt,curBar+1);

            for(j=1;j<=nCount;j++)
            {         
               if(mmtsLow > iLow(NULL,vInt,curBar+j)) 
               {
                  mmtsLow = iLow(NULL,vInt,curBar+j);
               }
            }
            for(j=1;j<=nCount;j++)
            { 
               if(mmtsHigh < iHigh(NULL,vInt,curBar+j)) 
               {
                  mmtsHigh = iHigh(NULL,vInt,curBar+j);
               }
            }
         
            mmtsLow     = mmtsLow - Slush;
            mmtsLow1    = mmtsLow - thick;
            mmtsHigh    = mmtsHigh + Slush;
            mmtsHigh1   = mmtsHigh + thick;     

            ObjectDelete("mmtsLow");
            ObjectCreate("mmtsLow",OBJ_RECTANGLE,0,vWhen,mmtsLow,vEnd,mmtsLow1);
            ObjectSet("mmtsLow",OBJPROP_COLOR,Yellow);
            ObjectSet("mmtsLow",OBJPROP_WIDTH,1);
            ObjectSet("mmtsLow",OBJPROP_STYLE,STYLE_DOT);       
     
            ObjectDelete("mmtsHigh");
            ObjectCreate("mmtsHigh",OBJ_RECTANGLE,0,vWhen,mmtsHigh,vEnd,mmtsHigh1);
            ObjectSet("mmtsHigh",OBJPROP_COLOR,Yellow);
            ObjectSet("mmtsHigh",OBJPROP_WIDTH,1);
            ObjectSet("mmtsHigh",OBJPROP_STYLE,STYLE_DOT);     
         }
      }
   }

}
*/

int doEntry(int curBar)
{
   double   vhigh, vlow, mid;
   return(0);
   vhigh = High[curBar+1];
   vlow  = Low[curBar+1];
   mid = ((vhigh - vlow) / 2) + vlow;
//   Comment(mid);

   return(0);
}